from micropython import const as con

ERROR = con(-1)
OK = con(1)

# https://stackoverflow.com/questions/1321878/how-to-prevent-favicon-ico-requests
# https://www.tutorialspoint.com/disable-browser-caching-with-meta-html-tags
# https://wiki.selfhtml.org/wiki/HTML/Elemente/meta#Automatische_Weiterleitung_zu_anderer_Adresse_.28Forwarding.29

HTML_HEAD = con("""
 <!DOCTYPE html>
 <head>
    <meta charset="utf-8" />
    <meta http-equiv="Expires" content="0" />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="icon" href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVQI12P4//8/AAX+Av7czFnnAAAAAElFTkSuQmCC">
    <title> LaserPeng </title>
 <head>
""")

HTML_HEAD_POLL = con("""
 <!DOCTYPE html>
 <head>
    <meta charset="utf-8" />
    <meta http-equiv="Expires" content="0" />
    <meta http-equiv="refresh" content="5" >
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="icon" href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVQI12P4//8/AAX+Av7czFnnAAAAAElFTkSuQmCC">
    <title> LaserPeng </title>
 <head>
""")

HTML_RINGS_STYLE = con("""     
     <style>
         body {text-align: center;}
         h1 {color: green;}
         a  {color: green;font-size: 1.8rem; font-weight: bold;}

         .circle {
             border: 1px solid black;
             border-radius: 50%;
             display:flex;
             align-items:center;
             justify-content:center;
             margin: auto;
             }
         .circle.large {
             width: 200px;
             height: 200px;                   
             }
          .circle.medium {
             width: 150px;
             height: 150px;                   
             }
         .circle.small {
             width: 100px;
             height: 100px;
             } 
         .circle.tiny {
             width: 30px;
             height: 30px;
             background-color: green; 
             }                
     </style>
""")

START_TARGET = con("Z")

TargetLedPins = con({   # GPIOs
    START_TARGET:  13,
    "A": 12,
    "B":  4,
    "C":  2,
    "D": 14
})

NpixPin = 5  # D1
NpixAllLeds = con((0, 56))

TargetRings = con({
    START_TARGET:  (NpixAllLeds[0], 8),  # Start ring
    "A":  (8, 20),
    "B":  (20, 32),
    "C":  (32, 44),
    "D":  (44, NpixAllLeds[1])
})

RgbColor = con({
    "Black":  (0, 0, 0),  # Led off
    "Red":  (25, 0, 0),
    "Yellow":  (25, 25, 0),
    "Blue":  (0, 0, 25),
    "Green":  (0, 13, 0),
    "Orange":  (25, 15, 0),
    "White":  (19, 19, 19)
})

ESP8266_WLAN = con({
    "SSID": "Laserpeng",
    "Password": "LaserPeng",
    "Port": 80
})

Misc = con({
    "ResultID": "55"  # Reaction time
})

TimeDef = con({
    "MasterTimeout":  30000,  # msec, max time to hit the Master
    "TargetTimeout":  20000,  # msec, max time to hit the Target
    "TargetStartDelay":  2000,  # msec, delay between master hit and start target hit
    "ShowTime": 5000,   # msec, Pop up screens e.g.Intro
    "TimeoutTag": -1
})

# Games
RandRing = con({    # Random rings
    "Name": "Zufallsringe",
    "ID": "81",
    "Rings2Shoot": 5
})

ColPatt = ({    # Color pattern
    "Name": "Farbmuster",
    "ID": "82",
    "Number": 5,
    "Color": 4,
})

RunPosi = ({    # Running position
    "Name": "Lfd. Keiler",
    "ID": "83",
    "Number": 5,
    "Color": 4,
})

# Dynamic webpage data
# ToDo: Put long term variables like page title in a different dictionary
DynHtmlData = {
    "RandRing_ID": RandRing["ID"],  # select game single color
    "RandRing_Name": RandRing["Name"],  # select game single color
    "ColPatt_ID": ColPatt["ID"],  # select game color pattern
    "ColPatt_Name": ColPatt["Name"],  # select game single color
    "RunPosi_ID": RunPosi["ID"],  # select game moving target, lfd. Keiler
    "RunPosi_Name": RunPosi["Name"],  # select game single color
    "Result_ID": Misc["ResultID"],  # get reaction time (sec)
    "SelectPageTitle": "Spiel wählen",
    "Ticks": "0.1",  # reaction time (msec)
    "LastResults": ["<<", "<<", "<<"]
}