# Ridolfa.eu / RolfS  2020 .. 2024

import gc
from time import sleep_ms, ticks_ms,  ticks_diff
import config as cfg
from webserver import WebSocket as WebSock
from html_pages import Pages as Html
from tools import NeoPixel, Util
import games


class BullsEye:
    def __init__(self):
        self.sock = None
        self.hitwait_startticks = None  # wait for a hit
        self.target_name = ""
        self.color_value = ()
        self.npx = NeoPixel(cfg.NpixPin, cfg.NpixAllLeds[1])
        self.util = Util(self.npx)
        self.websock = WebSock()
        self.html = Html()
        self.step = 1

    def get_game(self, request_id):
        game_obj = None
        if request_id == cfg.RandRing["ID"]:
            game_obj = games.RandRing(self.npx)
        elif request_id == cfg.ColPatt["ID"]:
            pass
        elif request_id == cfg.RunPosi["ID"]:
            pass
        elif request_id == cfg.Misc['ResultID']:
            print(32, cfg.DynHtmlData["Ticks"])
        else:
            game_obj = None
        return game_obj

    def step_01(self):
        """ Start a new game """
        print("\nStep.. ", self.step)
        self.npx.clear()
        cfg.DynHtmlData["Ticks"] = " --- "
        print(42, "--- Select a game ---")
        self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET], led=(0, 0), rgbcolor=cfg.RgbColor["Yellow"])
        self.step = 2

    def step_02(self):
        """ Show selection screen and wait for an action """
        print("\nStep.. ", self.step)
        request_id = self.websock.run_server(self.sock, self.html.game(), self.html.result())
        game_obj = self.get_game(request_id)
        self.step = 3
        return game_obj

    def step_03(self, game_obj):
        print("\nStep.. ", self.step)
        self.hitwait_startticks = game_obj.run()
        if self.hitwait_startticks == cfg.TimeDef["TimeoutTag"]:
            self.step = 4
        else:
            self.step = 5

    def step_04(self):
        print("\nStep.. ", self.step)
        self.util.success(False)
        print(66, "Timeout: ")
        cfg.DynHtmlData["Ticks"] = "Timeout"
        cfg.DynHtmlData["LastResults"].pop(0)
        cfg.DynHtmlData["LastResults"].append("TO")
        self.websock.run_server(self.sock, self.html.result())
        self.step = 6

    def step_05(self):
        print("\nStep.. ", self.step)
        self.util.success(True)
        reactiontime = "{:.2f}".format(ticks_diff(ticks_ms(), self.hitwait_startticks) / 1000)  # sec
        print(77, "Reaction time: ", reactiontime)
        cfg.DynHtmlData["Ticks"] = reactiontime
        cfg.DynHtmlData["LastResults"].pop(0)
        cfg.DynHtmlData["LastResults"].append(reactiontime)
        self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET], led=(4, 4),  # result indicator LED
                                rgbcolor=cfg.RgbColor["Blue"])
        self.websock.run_server(self.sock, self.html.result())
        self.step = 6

    def step_06(self):
        print("\nStep.. ", self.step)
        sleep_ms(cfg.TimeDef["ShowTime"])
        gc.collect()
        self.step = 1

    def play(self):
        game_obj = None
        while True:
            if self.step == 1:
                self.step_01()
            elif self.step == 2:
                game_obj = self.step_02()
            elif self.step == 3:
                self.step_03(game_obj)
            elif self.step == 4:
                self.step_04()
            elif self.step == 5:
                self.step_05()
            elif self.step == 6:
                self.step_06()

    def run(self):
        print(108, "------  P40 Laserpeng  --------")
        self.util.neopixel_intro(50)
        self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET], led=(0, 0), rgbcolor=cfg.RgbColor["Blue"])
        print(111, "Missing browser connection")
        self.websock.setup_network()
        self.sock = self.websock.setup_server_socket()
        self.websock.run_server(self.sock, self.html.intro())
        self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET], led=(0, 0), rgbcolor=cfg.RgbColor["Green"])
        print(116, "Browser connected")
        sleep_ms(cfg.TimeDef["ShowTime"])
        self.play()
