from time import sleep_ms, ticks_diff, ticks_ms
import neopixel
from machine import Pin
import config as cfg
from random import getrandbits


class NeoPixel:
    def __init__(self, npixpin, npixleds):
        self.npixleds = npixleds
        self.npix = neopixel.NeoPixel(Pin(npixpin), self.npixleds)

    def cycle(self, segment):  # cycle
        for i in range(segment[0], segment[1]):
            for j in range(self.npixleds):
                self.npix[j] = (0, 0, 0)
            self.npix[i % self.npixleds] = (255, 255, 255)
            self.npix.write()
            sleep_ms(25)
        return

    # def blink(self, segment, color=cfg.RgbColor["Green"], delay=400):
    #     # ToDo: extend cylcle as underlying method for blink
    #     for i in range(segment[0], 4 * segment[1]):
    #         for j in range(segment[0], segment[1]):
    #             self.npix[j] = color
    #         if (i // self.npixleds) % 2 == 0:
    #             self.npix[i % self.npixleds] = (0, 0, 0)
    #         else:
    #             self.npix[self.npixleds - 1 - (i % self.npixleds)] = (0, 0, 0)
    #         self.npix.write()
    #         sleep_ms(delay)

    def clear(self, pixs=None):
        """ Clear all Neopixel Leds """
        if pixs is None:
            pixs = self.npixleds
        for i in range(0, pixs):
            self.npix[i] = (0, 0, 0)
        self.npix.write()

    def npix_LEDstripe(self, segment, led=(0, 999), rgbcolor=cfg.RgbColor["Black"], ledsequence=1, delay=0):
        """
        Sets LED colors of a selected ring or all LEDs
        :param segment:     neopixel ring   e.g. ring[0]
        :param led:         tuple, min/max LED number within the segment   e.g  (3,3) only LED 3, (0,4) Led0  .. Led4
                            led(0,999): sets all LEDs of the ring ( assuming that maxLED < 999)
        :param rgbcolor:    e.g   cfg.RGB_Blue, cfg.RGB_Black=LED off
        :param ledsequence:  every nth LED, 1=every LED, 2=every second LED ,....
        :param delay  ms
        """
        maxLED = segment[1] - 1
        if led[1] < maxLED:
            maxLED = segment[0] + led[1]

        minLED = segment[0] + led[0]
        for x in range(minLED, maxLED + 1, ledsequence):
            self.npix[x] = rgbcolor
            self.npix.write()
            sleep_ms(delay)
        return

    # def ring_test(self, array, np):
    #     for x in range(array[0], array[1]):
    #         np[x] = (5, 25, 26)
    #         print(75, x)
    #         np.write()
    #         sleep_ms(100)
    #
    # def neopix_test(self):
    #     leds = 56
    #     np = neopixel.NeoPixel(Pin(5), leds)
    #     ring = ((0, 8), (8, 20), (20, 32), (32, 44), (44, leds))  # ZABCD
    #
    #     self.ring_test(ring[0], np)
    #     self.cycle(ring[0])
    #     self.clear(pixs=leds)


class Util:
    def __init__(self, npx):
        self.npx = npx

    def randint32(self, maxnum, nozero=False):
        """
        Returns a random number between 0 and maxnum.
        Limits: 2 < maxnum > 31 ,      Hint: 5 bits = 0 .. 31
        """
        if 2 < maxnum > 31:
            print(36, " randint32() error. maxnum out of limits [<2 or > 32]")
            return cfg.ERROR
        while True:
            number =  int(maxnum / 31 * getrandbits(5))
            if number == 0 and nozero:
                continue
            return number

    def random_dictkey(self, dic):
        """ Returns a random key/value pair of a dictionary"""
        key_list = list(dic.keys())
        key = key_list[self.randint32(len(dic)) - 1]
        value = dic[key]
        return key, value

    def random_target_ring(self):
        """ Returns a random target ring name and the led number range """
        target_name = cfg.START_TARGET
        target_value = None
        while target_name == cfg.START_TARGET:  # avoid to get the START Bull's Eye as target
            target_name, target_value = self.random_dictkey(cfg.TargetRings)
        return target_name, target_value

    def random_color(self):
        """ Returns a color key and the RGB values """
        color_value = cfg.RgbColor["Black"]
        color_key = None
        while color_value == cfg.RgbColor["Black"]:  # avoid to get Black as target ring color
            color_key, color_value = self.random_dictkey(cfg.RgbColor)
        # print(143, color_key, color_value,cfg.RgbColor["Black"])
        return color_key, color_value

    def wait_for_hit(self, pin, hitwait_startticks, timeout):
        """ Returns a hit ( True) or a timeout ( False)  """
        while pin.value():
            if ticks_diff(ticks_ms(), hitwait_startticks) > timeout:
                self.npx.clear()
                return False  # Timeout
        self.npx.clear()
        return True

    def shootsignal(self, on):
        if on:
            self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET],
                                    rgbcolor=cfg.RgbColor["White"], ledsequence=2, delay=2)
            return
        self.npx.clear()
        return

    def success(self, hit):
        for x in range(4):
            color = cfg.RgbColor["Green"]
            if not hit:
                color = cfg.RgbColor["Red"]
            self.npx.npix_LEDstripe(segment=cfg.TargetRings[cfg.START_TARGET],
                                    rgbcolor=color, ledsequence=2, delay=200)
            self.npx.clear()

    def setbit(self, bit, byte, bitvalue):
        """Changes a bit (0..7) in byte to the new bitvalue (0 or 1) and returns the new byte. """
        x = 1 << bit
        if bitvalue == 1:
            return byte | x
        return byte & (x ^ 0b11111111)  # 8 bit, 0xFF

    def neopixel_intro(self, delay):
        for i in cfg.TargetRings:
            color_key, color_value = self.random_color()
            self.npx.npix_LEDstripe(segment=cfg.TargetRings[i], rgbcolor=color_value, delay=delay)
            self.npx.clear()
